@@ -9,7 +9,6 @@ use strict;
 use warnings;
 use Carp 'croak';
 use JSON;
-use Fcntl;
 use IO::Select;
 use IO::Socket::SSL;
 use Time::HiRes qw(time);
@@ -18,13 +17,15 @@ use RPC::Switch::Client::Tiny::Netstring;
 use RPC::Switch::Client::Tiny::Async;
 use RPC::Switch::Client::Tiny::SessionCache;
 
-our $VERSION = '1.52_02';
+our $VERSION = '1.57_01';
 
 sub new {
 	my ($class, %args) = @_;
 	my $s = $args{sock} or croak __PACKAGE__ . " expects sock";
-	my $f = fcntl($s, F_GETFL, 0) or croak __PACKAGE__ . " bad socket: $!";
-	if ($f & O_NONBLOCK) { croak __PACKAGE__ . " nonblocking socket not supported"; }
+	unless ($^O eq 'MSWin32') { # cpantester: strawberry perl does not support blocking() call
+		defined(my $b = $s->blocking()) or croak __PACKAGE__ . " bad socket: $!";
+		unless ($b) { croak __PACKAGE__ . " nonblocking socket not supported"; }
+	}
 	unless (exists $args{who}) { croak __PACKAGE__ . " expects who"; }
 	my $self = bless {
 		%args,
@@ -41,6 +42,7 @@ sub new {
 	} else {
 		$self->{auth_method} = 'password' unless exists $self->{auth_method};
 	}
+	$self->{json_utf8} = $self->{client_encoding_utf8} ? {} : {utf8 => 1};
 	return $self;
 }
 
@@ -52,7 +54,7 @@ sub rpc_send {
 	my ($self, $msg) = @_;
 	my $s = $self->{sock};
 	$msg->{jsonrpc} = '2.0';
-	my $str = to_json($msg, {canonical => 1});
+	my $str = to_json($msg, {canonical => 1, %{$self->{json_utf8}}});
 	$self->{trace_cb}->('SND', $msg) if $self->{trace_cb};
 	return netstring_write($s, $str);
 }
@@ -344,6 +346,8 @@ sub child_handler {
 	}
 	$self->{sock} = $wr;
 	delete $self->{trace_cb};
+	local $SIG{INT} = 'DEFAULT';
+	local $SIG{PIPE} = 'IGNORE'; # handle sigpipe via print/write result
 
 	# When session handling is enabled a child might process
 	# more than one request with the same session_id.
@@ -355,7 +359,7 @@ sub child_handler {
 			die "worker child: $@" if $@;
 			last; # EOF
 		}
-		my $msg = eval { from_json($b) };
+		my $msg = eval { from_json($b, {%{$self->{json_utf8}}}) };
 		die "worker child: $@" if $@;
 
 		# The client catches all possible die() calls, so that it is
@@ -363,13 +367,17 @@ sub child_handler {
 		#
 		my $params;
 		my $callback = $self->{methods}{$msg->{method}}{cb};
-		my @resp = eval { $callback->($msg->{params}, $msg->{rpcswitch}) };
+		my @resp;
+		eval {
+			local $SIG{PIPE} = 'DEFAULT'; # reenable sigpipe for worker code
+			@resp = $callback->($msg->{params}, $msg->{rpcswitch});
+		};
 		if (my $err = $@) {
 			$params = ['RES_ERROR', $msg->{id}, $err];
 		} else {
 			$params = ['RES_OK', $msg->{id}, @resp];
 		}
-		$b = eval { to_json($params) };
+		$b = eval { to_json($params, {%{$self->{json_utf8}}}) };
 		return 1 if $@; # signal die from json encode
 		my $res = netstring_write($self->{sock}, $b);
 		return 2 unless $res; # signal socket error
@@ -383,7 +391,7 @@ sub child_handler {
 sub _worker_child_write {
 	my ($self, $child, $msg) = @_;
 
-	my $b = to_json($msg, {canonical => 1});
+	my $b = to_json($msg, {canonical => 1, %{$self->{json_utf8}}});
 	my $res = netstring_write($child->{reader}, $b); # forward request to worker child
 	die rpc_error('io', 'netstring_write') unless $res;
 	return;
@@ -436,7 +444,7 @@ sub _worker_child_read_and_finish {
 		$res = $self->rpc_send({method => 'rpcswitch.result', params => ['RES_ERROR', $child->{id}, $err], rpcswitch => $child->{rpcswitch}});
 		$self->{async}->child_finish($child, 'error');
 	} else {
-		my $params = eval { from_json($b) };
+		my $params = eval { from_json($b, {%{$self->{json_utf8}}}) };
 		if ($@) {
 			$res = $self->rpc_send({method => 'rpcswitch.result', params => ['RES_ERROR', $child->{id}, $@], rpcswitch => $child->{rpcswitch}});
 			$self->{async}->child_finish($child, 'error');
@@ -545,7 +553,7 @@ sub rpc_handler {
 			die rpc_error('io', $@) if $@;
 			return; # EOF
 		}
-		my $msg = eval { from_json($b) };
+		my $msg = eval { from_json($b, {%{$self->{json_utf8}}}) };
 		die rpc_error('jsonrpc', $@) if $@;
 		$self->{trace_cb}->('RCV', $msg) if $self->{trace_cb};
 		my $res = eval { $handler->($self, $msg, @handler_params) };
@@ -742,11 +750,11 @@ of the rpc-switch server.
 
 The implementation is based on the following protocols:
 
-- json-rpc 2.0: https://www.jsonrpc.org/specification
+- json-rpc 2.0: L<https://www.jsonrpc.org/specification>
 
-- netstring proto: http://cr.yp.to/proto/netstrings.txt
+- netstring proto: L<http://cr.yp.to/proto/netstrings.txt>
 
-- rpc-switch: https://github.com/a6502/rpc-switch
+- rpc-switch: L<https://github.com/a6502/rpc-switch>
 
 =head2 Error Handling
 
@@ -770,13 +778,18 @@ on the error type:
 
 =head2 Encoding
 
-All methods expect and return strings in utf8-format.
+All methods expect and return strings in utf8-format, when the option
+client_encoding_utf8 is enabled.
 
 To pass latin1-strings as parameter, a caller would have to convert
 the input first. (see: $utf8 = Encode::encode('utf8', $latin1)).
 
 The json-rpc messages are also utf8-encoded when they are transmitted.
-(see: https://metacpan.org/pod/JSON#utf8)
+(see: L<https://metacpan.org/pod/JSON#utf8>)
+
+Without the client_encoding_utf8 option, all passed strings are utf8-encoded
+for the json-rpc transmission, because the client character-encoding is
+assumed to be unknown. This might transmit doubly utf8-encoded strings.
 
 =head2 Async handling
 
@@ -800,7 +813,7 @@ one request with the same session_id.
 
 The session handling is loosely based on the HTTP Set-Cookie/Cookie
 Mechanism described in RFC 6265, and uses json paramters instead of
-http-headers. (see: https://datatracker.ietf.org/doc/html/rfc6265)
+http-headers. (see: L<https://datatracker.ietf.org/doc/html/rfc6265>)
 
 - the worker signals a set_session request via:
 
@@ -890,6 +903,8 @@ The accepted arguments are:
 
 =item token: token for auth_method 'password' (optional for 'clientcert')
 
+=item client_encoding_utf8: all client strings are in utf8-format
+
 =item timeout: optional rpc-request timeout (defaults to 0 (unlimited))
 
 =item trace_cb: optional handler to log SND/RCV output
@@ -956,6 +971,8 @@ connection, and there are no outstanding requests ($@ == '' for eval).
 
 =item options->{session_expire}: default session expire time in seconds
 
+=item options->{max_user_session}: limit session cache per optional user field
+
 =back
 
 The options are described in their respective sections unter DESCRIPTION.
@@ -968,11 +985,11 @@ The valid fields of the $method_definition are:
 
 =over 4
 
-=item - cb: method handler (required)
+=item cb: method handler (required)
 
-=item - doc: optional documentation for method
+=item doc: optional documentation for method
 
-=item - filter: optional filter to restrict method subset of params
+=item filter: optional filter to restrict method subset of params
 
 =back
 
